/*******************************************************************************
Copyright © 2023 Sebastian Roith

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*******************************************************************************/

window.TASK = function(fktRandomInt) {
    function getUniqueValues(count, min, max) {
        const values = [];
        while (values.length < count) {
            const randomValue = fktRandomInt(min, max - values.length);
            values.push(values.slice().sort((a, b) => a - b).reduce((newValue, oldValue) => ((newValue >= oldValue) ? (newValue + 1) : newValue), randomValue));
        }
        return values;
    }

    const result = {
        task: 'Führe den Code aus, indem du immer die Auswirkungen der hervorgehobenen Zeile in der Speichertabelle einträgst und die Codezeile auswählst, welche die nächste auszuführende <strong>Anweisung</strong> enthält.',
        code: '',
        notSelectable: [],
        steps: []
    }

    const varType = 'int';
    const varNames = getUniqueValues(3, 'a'.charCodeAt(0), 'z'.charCodeAt(0)).map(code => String.fromCharCode(code));
    const varValues = getUniqueValues(3, 0, 5);
    const loopsVarIndex = fktRandomInt(0, 2);
    const countLoops = fktRandomInt(2, 4);
    const monotonUps = [
        { random: [], code: (x, y) => (x + ' = ' + x + ' + ' + y), exec: (x, y) => (x + y) },
        { random: [], code: (x, y) => (x + ' = ' + y + ' + ' + x), exec: (x, y) => (x + y) },
        { random: [[1, 5]], code: (x, y, plus) => (x + ' = ' + x + ' + ' + plus), exec: (x, y, plus) => (x + plus) },
        { random: [[2, 4]], code: (x, y, mal) => (x + ' = ' + x + ' * ' + mal), exec: (x, y, mal) => (x * mal) },
        { random: [[1, 5]], code: (x, y, plus) => (x + ' = ' + plus + ' + ' + x), exec: (x, y, plus) => (x + plus) },
        { random: [[2, 4]], code: (x, y, mal) => (x + ' = ' + mal + ' * ' + x), exec: (x, y, mal) => (x * mal) }
        /*!*/
    ];
    const rawAdapt = { random: [], code: (x, y) => (x + ' = ' + y), exec: (x, y) => y };
    function generateUp(raw, index1, index2) {
        const random = raw.random.map(([min, max]) => fktRandomInt(min, max));
        return {
            code() { return raw.code(varNames[index1], varNames[index2], ...random) + ';'; },
            exec() { current[index1] = raw.exec(current[index1], current[index2], ...random); }
        };
    }

    let current = [];
    let allowAdaptValue = true;
    const ups = varNames.filter((varName, index) => (index != loopsVarIndex)).map(varName => {
        const adaptValue = (allowAdaptValue && !fktRandomInt(0, 2));
        if (adaptValue)
            allowAdaptValue = false;
        const fktUp = adaptValue ? rawAdapt : monotonUps[fktRandomInt(0, monotonUps.length - 1)];
        const varIndex = varNames.indexOf(varName);
        const otherIndex = varNames.map((name, index) => index).filter(index => (index != varIndex))[fktRandomInt(0, varNames.length - 2)];
        return generateUp(fktUp, varIndex, otherIndex);
    });
    if (fktRandomInt(0, 1))
        ups.reverse();

    function genVars() {
        return current.map((value, index) => ({ key: varNames[index], value, type: varType }));
    }

    function getNextLine() {
        return (result.code.match(/\n/g) || []).length + 1;
    }

    function addVar() {
        const index = current.length;
        if (index >= varNames.length)
            return;
        result.code += (result.code.length ? '\n' : '') + varType + ' ' + varNames[index] + ' = ' + varValues[index] + ';';
        current.push(varValues[index]);
        result.steps.push({ line: getNextLine(), vars: genVars() });
    }

    result.steps.push({ line: 0, vars: [] });
    varNames.forEach(() => addVar());

    result.notSelectable.push(getNextLine());
    result.notSelectable.push(getNextLine() + 1);
    result.code += '\n\nfor (';
    result.steps[result.steps.length - 1].line = getNextLine();

    result.code += '\n    ' + varNames[loopsVarIndex] + ' = 0;';
    const indexCondition = getNextLine();
    current[loopsVarIndex] = 0;
    result.steps.push({ line: indexCondition, vars: genVars() });

    result.code += '\n    ' + varNames[loopsVarIndex] + ' < ' + countLoops + ';';
    const indexStepUp = getNextLine();
    current[loopsVarIndex] = 0;
    result.steps.push({ line: indexStepUp + 2, vars: genVars() });

    result.code += '\n    ' + varNames[loopsVarIndex] + ' = ' + varNames[loopsVarIndex] + ' + 1\n) {';
    const indexLoopStart = getNextLine();
    result.notSelectable.push(indexLoopStart - 1);
    ups.forEach(up => (result.code += '\n    ' + up.code()));
    result.notSelectable.push(getNextLine());
    result.code += '\n}';

    for (let loopRun = 1; loopRun <= countLoops; ++loopRun) {
        ups.forEach((up, index) => {
            up.exec();
            result.steps.push({ line: indexLoopStart + index + 1, vars: genVars() });
        });
        result.steps[result.steps.length - 1].line = indexStepUp;
        ++current[loopsVarIndex];
        result.steps.push({ line: indexCondition, vars: genVars() });
        result.steps.push({ line: (loopRun < countLoops) ? indexLoopStart : getNextLine(), vars: genVars() });
    }
    const lastUpsVarIndizes = getUniqueValues(2, 0, varNames.length - 1);
    const lastUp = generateUp(fktRandomInt(0, 3) ? monotonUps[fktRandomInt(0, monotonUps.length - 1)] : rawAdapt, ...lastUpsVarIndizes);
    result.code += '\n' + lastUp.code();
    lastUp.exec();
    result.steps.push({ line: Infinity, vars: genVars() });

    return result;
};
