function VarTypeFloat(
    table,      // Responsible TableOfVars instance
    varName     // Name of this variable
) {
    VarType.call(this, table, varName);
}
VarTypeFloat.prototype = Object.create(VarType.prototype);
VarTypeFloat.prototype.constructor = VarTypeFloat;

/** Add to list of available types */
TableOfVars.varTypes.push(VarTypeFloat);

/** Name of variable type */
VarTypeFloat.typeName = 'float';
VarTypeFloat.prototype.typeName = 'float';

/** Used number of bytes to store this variable */
VarTypeFloat.countBytes = 4;

/** Default value of variables of this type */
VarTypeFloat.defaultValue = 0;

/** Present value */
Object.defineProperty(VarTypeFloat.prototype, 'value', { enumerable: true,
    get() { return +this.input.value; },
    set(v) {
        if (Number.isFinite(v))
            this.input.value = v;
    }
});

/**
* Create a input for this type
* @return HTMLElement       Created input, select, …
*/
VarTypeFloat.prototype._createInput = function(
    document        // HTMLDocument instance to use
) {
    const input = document.createElement('input');
    input.type = 'number';
    input.value = VarTypeFloat.defaultValue;
    return input;
}

/** Accepted tollerance when comparing values */
VarTypeFloat.prototype.precision = 0.001;

/**
* Check if variable's value is given value
* @return Boolean       Is variable's value given value?
*/
VarTypeFloat.prototype.equals = function(
    value       // Value to compare with
) {
    return (Math.abs(this.value - value) < this.precision);
}
